{% load i18n %}

<div class="pagination">
  {% for i in page_obj.paginator.page_range %}
    {% if i == page_obj.paginator.ELLIPSIS %}
      <span>...</span>
    {% elif i == page_obj.number %}
      <span class="current">{{ i }}</span>
    {% else %}
      <a href="?{{ page_var }}={{ i }}">{{ i }}</a>
    {% endif %}
  {% endfor %}
</div>
<p class="pagination-info">
  {{ page_obj.paginator.count }} {% blocktranslate count counter=page_obj.paginator.count %}entry{% plural %}entries{% endblocktranslate %}
</p>
